<?php
/**
 * @file
 * restmini: Admin layer.
 */


/**
 * Displays overview of standard REST HTTP methods and responses.
 *
 * @return string
 */
function restmini_admin_help() {
  drupal_add_css(
    drupal_get_path('module', 'restmini') . '/admin/restmini.admin.css',
    array('type' => 'file', 'group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );

  $table_header = array(
    '', // Who's fault.
    array(
      'data' => t('HTTP Request methods !x', array('!x' => '&nbsp; &#x25b6;'), array('context' => 'module:restmini')),
      'colspan' => 2,
      'class' => array('restmini-help-capitalize'),
    ),
    '<b>HEAD</b>',
    '<b>GET</b>',
    '<b>POST</b>',
    '<b>PUT</b>',
    '<b>DELETE</b>',
  );
  $table_rows = array();

  $table_rows[] = array(
    'data' => array(
      array( // Who's fault.
        'data' => '',
        'colspan' => 2,
        'rowspan' => 2,
        'class' => array('restmini-help-odd-even'),
      ),
      array(
        'data' => t('Operations', array(), array('context' => 'module:restmini')),
        'rowspan' => 2,
        'class' => array('restmini-help-odd-even', 'restmini-help-capitalize', 'restmini-help-bold', 'restmini-help-border-right'),
      ),
      t('!bindex!_b!nllist all objects', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
      array(
        'data' => t('!bindex!_b!nllist all objects', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
        'class' => array('restmini-help-border-right'),
      ),
      array(
        'data' => t('!bcreate!_b!nlone or more objects!nlwith request body', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
        'rowspan' => 2,
      ),
      array(
        'data' => t('!bupdate!_b!nlone or more objects!nlby a path parameter!nland request body', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
        'rowspan' => 2,
        'class' => array('restmini-help-border-right'),
      ),
      array(
        'data' => t('!bdelete!_b!nlone or more objects!nlby a path parameter', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
        'rowspan' => 2,
      ),
    ),
  );

  $table_rows[] = array(
    'data' => array(
      t('!bretrieve!_b!nlone or more objects!nlby a path parameter', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
      array(
        'data' => t('!bretrieve!_b!nlone or more objects!nlby a path parameter', array('!nl' => '<br/>', '!b' => '<b>', '!_b' => '</b>'), array('context' => 'module:restmini')),
        'class' => array('restmini-help-border-right'),
      ),
    ),
  );

  $table_rows[] = array(
    'data' => array(
      array( // Who's fault.
        'data' => t('Outcome', array(), array('context' => 'module:restmini')),
        'class' => array('restmini-help-inner-legend', 'restmini-help-border-right'),
      ),
      array(
        'data' => t('Response status !y', array('!y' => '&nbsp; &#x25bc;'), array('context' => 'module:restmini')),
        'class' => array('restmini-help-inner-legend', 'restmini-help-capitalize'),
      ),
      array(
        'data' => t('Notes', array(), array('context' => 'module:restmini')),
        'class' => array('restmini-help-inner-legend', 'restmini-help-border-right'),
      ),
      array(
        'data' => t('•&nbsp; Response body - if request method&mdash;response status combination allowed at all &nbsp;•',
          array(), array('context' => 'module:restmini')),
        'colspan' => 5,
        'class' => array('restmini-help-inner-legend', 'restmini-help-center'),
      ),
    ),
  );


  $status_methods = RestMini::statusCodes();

  // Count 2xx + 3xx, 4xx and 5xx.
  $status_codes = array_keys($status_methods);
  $benign = $client = $server = 0;
  foreach ($status_codes as $code) {
    if ($code < 400) {
      ++$benign;
    }
    elseif ($code < 500) {
      ++$client;
    }
    else {
      ++$server;
    }
  }

  foreach ($status_methods as $code => $methods) {
    $note = '';
    switch ($code) {
      case 201:
        $note = t('when PUT: Updated', array(), array('context' => 'module:restmini'));
        break;
      case 204:
        $note = t('silent success', array(), array('context' => 'module:restmini'));
        break;
      case 304:
        $note = t('client use cache instead', array(), array('context' => 'module:restmini'));
        break;
      case 400:
        $note = t('a parameter!nlfailed validation', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 404:
        $note = t('no such endpoint!nl- OR - no such object', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 405:
        $note = t('endpoint doesn\'t support!nlMETHOD; Allow header', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 406:
        $note = t('endpoint doesn\'t support!nlrequested content type', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 410:
        $note = t('doesn\'t exist any more', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 415:
        $note = t('not application/-!nlx-www-form-urlencoded', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 500:
        $note = t('if RESTmini Service self:!nlbad router cache', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 501:
        $note = t('...yet', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 502:
        $note = t('a background (proxied)!nl service failed', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 503:
        $note = t('if RESTmini Service self:!nlincomplete endpoint', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
      case 504:
        $note = t('request to a proxied!nlservice timed out', array('!nl' => '<br/>'), array('context' => 'module:restmini'));
        break;
    }

    $row = array(
      array(
        'data' => '<b>' . $code . '</b>&emsp;' . RestMini::statusText($code),
        'class' => array(''),
      ),
      array(
        'data' => $note,
        'class' => array('restmini-help-comment', 'restmini-help-border-right'),
      ),
      array(
        'data' => in_array('HEAD', $methods) ? t('no', array(), array('context' => 'module:restmini')) : '',
        'class' => array(in_array('HEAD', $methods) ? 'restmini-help-body-no' : ''),
      ),
      array(
        'data' => in_array('GET', $methods) ?
          ($code == 200 ? t('yes', array(), array('context' => 'module:restmini')) :
            ($code < 400 ? t('no', array(), array('none' => 'module:restmini')) :
              t('optional', array(), array('context' => 'module:restmini')))) : '',
        'class' => array('restmini-help-border-right',
          in_array('GET', $methods) ? ('restmini-help-body-' . ($code == 200 ? 'yes' : ($code < 400 ? 'no' : 'optional'))) : ''),
      ),
      array(
        'data' => in_array('POST', $methods) ?
          ($code <= 201 ? t('yes', array(), array('context' => 'module:restmini')) :
            ($code < 400 ? t('no', array(), array('none' => 'module:restmini')) :
              t('optional', array(), array('context' => 'module:restmini')))) : '',
        'class' => array(in_array('POST', $methods) ? ('restmini-help-body-' . ($code <= 201 ? 'yes' : ($code < 400 ? 'no' : 'optional'))) : ''),
      ),
      array(
        'data' => in_array('PUT', $methods) ?
          ($code <= 201 ? t('yes', array(), array('context' => 'module:restmini')) :
            ($code < 400 ? t('no', array(), array('none' => 'module:restmini')) :
              t('optional', array(), array('context' => 'module:restmini')))) : '',
        'class' => array('restmini-help-border-right',
          in_array('PUT', $methods) ? ('restmini-help-body-' . ($code <= 201 ? 'yes' : ($code < 400 ? 'no' : 'optional'))) : ''),
      ),
      array(
        'data' => in_array('DELETE', $methods) ?
          ($code == 200 ? t('yes', array(), array('context' => 'module:restmini')) :
            ($code < 400 ? t('no', array(), array('none' => 'module:restmini')) :
              t('optional', array(), array('context' => 'module:restmini')))) : '',
        'class' => array(in_array('DELETE', $methods) ? ('restmini-help-body-' . ($code == 200 ? 'yes' : ($code < 400 ? 'no' : 'optional'))) : ''),
      ),
    );

    switch ($code) {
      case 200:
        array_unshift($row, array(
          'data' => t('unconditional!nlsuccess', array('!nl' => '<br/>'), array('context' => 'module:restmini')),
          'rowspan' => $benign,
          'class' => array('restmini-help-odd-even'),
        ));
        break;
      case 400:
        array_unshift($row, array(
          'data' => t('client!nlfailed', array('!nl' => '<br/>'), array('context' => 'module:restmini')),
          'rowspan' => $client,
          'class' => array('restmini-help-fault-client'),
        ));
        break;
      case 500:
        array_unshift($row, array(
          'data' => t('service!nlendpoint!nlfailed', array('!nl' => '<br/>'), array('context' => 'module:restmini')),
          'rowspan' => $server,
          'class' => array('restmini-help-fault-server'),
        ));
        break;
    }

    $table_rows[] = array(
      'data' => $row,
    );
  }

  return theme_table(
    array(
      'header' => $table_header,
      'rows' => $table_rows,
      'attributes' => array(
        'class' => array(
          'restmini',
          'restmini-help',
        ),
      ),
      'caption' => '',
      'colgroups' => array(),
      'sticky' => TRUE,
      'empty' => t(
        'No data',
        array(),
        array('context' => 'module:restmini')
      ),
    )
  );
}

/**
 * Displays overview of validation patterns and checks.
 *
 * @return string
 */
function restmini_admin_validation() {
  drupal_add_css(
    drupal_get_path('module', 'restmini') . '/admin/restmini.admin.css',
    array('type' => 'file', 'group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );

  $output = '';

  $output .= '<h3>' . t('Validation patterns - types and applicable rules', array(), array('context' => 'module:restmini:validation')) . '</h3>'
   . '<p>' . t(
      'Every pattern has a \'type\' rule which defines the expected data type of the actual value.',
      array(), array('context' => 'module:restmini:validation')
    ) . '</p>';

  $table_header = array(
    t('Rule name', array(), array('context' => 'module:restmini:validation')),
    t('Rule description', array(), array('context' => 'module:restmini:validation')),
    t('Expected <b>type</b> !x', array('!x' => '&nbsp; &#x25b6;'), array('context' => 'module:restmini')),
  );
  $types = array(
    'integer',
    'float',
    'string',
    'array',
    'object',
    'boolean',
    'NULL',
  );
  foreach ($types as $val) {
    $table_header[] = '<b>' . $val . '</b>';
  }

  $ruleDescriptions = array(
    'pattern' => t('(string) Merge this pattern with a predefined pattern of that name.', array(), array('context' => 'module:restmini:validation')),
    'extends' => t('(array) Internal flag: current pattern has been merged<br>&nbsp; &nbsp; with that/these pattern(s).', array(), array('context' => 'module:restmini:validation')),
    'title' => t('(string) Rule title/description.', array(), array('context' => 'module:restmini:validation')),
    'type' => t('(string) Default: \'string\'. Expect a value of that type.', array(), array('context' => 'module:restmini:validation')),
    'optional' => t('(boolean) Default: FALSE.', array(), array('context' => 'module:restmini:validation')),
    'default' => t('(mixed) If not set, or if empty/no value: set that value.*', array(), array('context' => 'module:restmini:validation')),
    'except_type' => t('(string) If that and actual data type are the same, then no type<br>&nbsp; &nbsp; mismatch. Must be \'array\'|\'object\'|\'boolean\'|\'NULL\'.', array(), array('context' => 'module:restmini:validation')),
    'except_value' => t('(string) If that and actual value are the same, then validation<br>&nbsp; &nbsp; is passed. Cannot be array or object.', array(), array('context' => 'module:restmini:validation')),
    'no_trim' => t('(boolean) Default: FALSE; do remove leading and traling spaces.*<br>&nbsp; &nbsp; Trimming is applied <em>before</em> length evaluation.', array(), array('context' => 'module:restmini:validation')),
    'min_length' => t('(string) Minimum length.', array(), array('context' => 'module:restmini:validation')),
    'max_length' => t('(string) Maximum length.', array(), array('context' => 'module:restmini:validation')),
    'exact_length' => t('(string) Exact length; ignore minimum and maximum length.', array(), array('context' => 'module:restmini:validation')),
    'truncate' => t('(integer) Shorten string to that length.*<br>&nbsp; &nbsp; Truncation is applied <em>after</em> length evaluation.', array(), array('context' => 'module:restmini:validation')),
    'min' => t('(integer) Minimum value.', array(), array('context' => 'module:restmini:validation')),
    'max' => t('(integer) Maximum value.', array(), array('context' => 'module:restmini:validation')),
    'enum' => t('(array) List of valid values (cannot be combined with check or regex).', array(), array('context' => 'module:restmini:validation')),
    'check' => t('(string) Name of predefined check to test value against.', array(), array('context' => 'module:restmini:validation')),
    'regex' => t('(string) A regular expression delimited by forward slash.', array(), array('context' => 'module:restmini:validation')),
  );

  $rulesAll = array(
    'pattern',
    'extends',
    'title',
    'type',
    'optional',
    'default',
    'except_type',
    'except_value',
  );
  $rulesForTypes = array(
    'NULL' => $rulesAll,
    'boolean' => $rulesAll,
    'integer' => array_merge($rulesAll, array('min_length', 'max_length', 'exact_length', 'enum', 'check', 'regex', 'min', 'max')),
    'float' => array_merge($rulesAll, array('min_length', 'max_length', 'exact_length', 'enum', 'check', 'regex', 'min', 'max')),
    'string' => array_merge($rulesAll, array('min_length', 'max_length', 'exact_length', 'enum', 'check', 'regex', 'no_trim', 'truncate')),
    'array' => $rulesAll,
    'object' => $rulesAll,
  );

  $table_rows = array();
  foreach ($ruleDescriptions as $ruleName => $description) {
    $row = array(
      'data' => array(
        '<b>' . $ruleName . '</b>',
        array(
          'data' => $description,
          'colspan' => 2,
        ),
      ),
    );
    foreach ($types as $type) {
      $continue = FALSE;
      if (in_array($ruleName, $rulesForTypes[$type])) {
        $column = array(
          'data' => '&nbsp;',
          'class' => array('restmini-center restmini-validation-yes'),
        );

        switch ($ruleName) {
          case 'pattern':
          case 'extends':
          case 'title':
            if ($ruleName == 'pattern' && $type == 'integer') {
              $column['data'] = t('type-independent', array(), array('context' => 'module:restmini:validation'));
              $column['colspan'] = 7;
              $column['rowspan'] = 3;
            }
            else {
              $continue = TRUE;
            }
            break;
        }

        switch ($type) {
          case 'integer':
          case 'float':
            switch ($ruleName) {
              case 'type':
                $column['data'] = t('string gets cast to !type*', array('!type' => $type), array('context' => 'module:restmini:validation'));
                break;
              case 'optional':
                if ($type == 'integer') {
                  $column['data'] = t('empty string is empty/no value - 0 and \'0\' is not', array(), array('context' => 'module:restmini:validation'));
                  $column['colspan'] = 3;
                }
                else {
                  // type:integer colspan.
                  $continue = TRUE;
                }
                break;
              case 'enum':
                $column['data'] = t('type-safe comparison', array(), array('context' => 'module:restmini:validation'));
                break;
            }
            break;
          case 'string':
            switch ($ruleName) {
              case 'type':
                $column['data'] = t('number gets cast to string*', array(), array('context' => 'module:restmini:validation'));
                break;
              case 'optional':
                // type:integer colspan.
                $continue = TRUE;
                break;
            }
            break;
          case 'array':
          case 'object':
            switch ($ruleName) {
              case 'optional':
                $column['data'] = t('empty !type is empty/no value', array('!type' => $type), array('context' => 'module:restmini:validation'));
                break;
            }
            break;
        }
        if ($continue) {
          continue;
        }
      }
      else {
        $column = array(
          'data' => t('n/a', array(), array('context' => 'module:restmini')),
          'class' => array('restmini-validation-no'),
        );
      }

      $row['data'][] = $column;
    }

    $table_rows[] = $row;
  }

  $output .= theme_table(
    array(
      'header' => $table_header,
      'rows' => $table_rows,
      'attributes' => array(
        'class' => array(
          'restmini',
          'restmini-validation',
        ),
      ),
      'caption' => '',
      'colgroups' => array(),
      'sticky' => TRUE,
      'empty' => t(
        'No data',
        array(),
        array('context' => 'module:restmini')
      ),
    )
  )
  . '<p class="restmini-validation-notes">'
  . t('* May alter the actual value.', array(), array('context' => 'module:restmini_service'))
  . '</p>';


  $patterns = RestMini::validationPattern();
  $output .= '<h3>' . t('Predefined validation patterns', array(), array('context' => 'module:restmini:validation')) . '</h3>'
    . '<p>' . t(
      'You can use any of these patterns as-is, or define a custom ad hoc pattern.<br>Or define additional patterns by implementing hook_restmini_validation_patterns().<br>An ad hoc pattern as well as an addional pattern may extend a predefined pattern by using the \'pattern\' rule.<br>Beware that predefined validation patterns are cached - you must clear cache to use a newly defined pattern.',
      array(), array('context' => 'module:restmini:validation')
    ) . '</p>';
  $table_header = array(
    t('Name', array(), array('context' => 'module:restmini:validation')),
    t('Title', array(), array('context' => 'module:restmini:validation')),
    t('Type', array(), array('context' => 'module:restmini:validation')),
    t('Extends', array(), array('context' => 'module:restmini:validation')),
    t('Rules', array(), array('context' => 'module:restmini:validation')),
  );
  $table_rows = array();
  foreach ($patterns as $name => $rules) {
    $row = array(
      '<b>' . $name . '</b>',
      array(
        'data' => $rules['title'],
        'class' => array('restmini-validation-pattern-title'),
      ),
      $rules['type'],
      empty($rules['extends']) ? '-' : join(', ', $rules['extends']),
    );
    unset($rules['pattern'], $rules['optional'], $rules['prepared'], $rules['title'], $rules['type'], $rules['extends']);
    $stringed = array();
    foreach ($rules as $rule_name => $val) {
      $s = $rule_name . ': ';
      switch (gettype($val)) {
        case 'string':
          $s .= '\'' . $val . '\'';
          break;
        case 'integer':
          $s .= '(integer) ' . $val;
          break;
        case 'float':
        case 'double':
          $s .= '(float) ' . $val;
          break;
        case 'boolean':
          $s .= '(boolean) ' . ($val ? 'TRUE' : 'FALSE');
          break;
        case 'NULL':
          $s .= '(NULL) NULL';
          break;
        case 'array':
          $s .= 'array(' . count($val) . ')';
          break;
        case 'object':
          $s .= 'object(' . count(get_object_vars($val)) . ')';
          break;
        default:
          $s .= '(unknown type)';
      }
      $stringed[] = $s;
    }
    $row[] = join('<br>', $stringed);
    $table_rows[] = $row;
  }
  $output .= theme_table(
    array(
      'header' => $table_header,
      'rows' => $table_rows,
      'attributes' => array(
        'class' => array(
          'restmini',
          'restmini-validation',
        ),
      ),
      'caption' => '',
      'colgroups' => array(),
      'sticky' => TRUE,
      'empty' => t(
        'No data',
        array(),
        array('context' => 'module:restmini')
      ),
    )
  );


  module_load_include('inc', 'restmini', 'restmini_validation_check');
  $checks = restmini_validation_check();

  $output .= '<p>&nbsp;</p>'
    . '<h3>' . t('Validation checks', array(), array('context' => 'module:restmini:validation')) . '</h3>'
    . '<p>' . t(
      'A \'check\' is a predefined algo for testing a string or stringed numeric value. It may be a regular expression, or a more complex but still compact procedure.<br>If a validation pattern has a \'check\' rule, that check will be executed during validation (unless the value fails or passes due to some other pattern rule).',
      array(), array('context' => 'module:restmini:validation')
    ) . '</p>';
  $table_header = array(
    t('Check name', array(), array('context' => 'module:restmini:validation')),
    t('Regular expression or description', array(), array('context' => 'module:restmini:validation')),
  );
  $table_rows = array();
  foreach ($checks as $name => $value) {
    $table_rows[] = array(
      '<b>' . $name . '</b>',
      $value,
    );
  }
  $output .= theme_table(
    array(
      'header' => $table_header,
      'rows' => $table_rows,
      'attributes' => array(
        'class' => array(
          'restmini',
          'restmini-validation',
        ),
      ),
      'caption' => '',
      'colgroups' => array(),
      'sticky' => TRUE,
      'empty' => t(
        'No data',
        array(),
        array('context' => 'module:restmini')
      ),
    )
  );

  return $output;
}
